@@ -26,20 +26,25 @@ use warnings;
 use utf8;
 use autodie;
 
+use Cwd qw(getcwd);
 use File::Basename;
+use IO::Async::Loop;
+use IO::Async::Process;
 use IO::Uncompress::Gunzip qw(gunzip $GunzipError);
 use List::Compare;
 use List::MoreUtils qw(any none);
+use Path::Tiny;
 use Text::ParseWords ();
 use Unicode::UTF8 qw(valid_utf8 decode_utf8);
 
 use Lintian::Spelling qw(check_spelling);
-use Lintian::Util qw(clean_env do_fork drain_pipe open_gz);
+use Lintian::Util qw(open_gz);
 
 use constant LINTIAN_COVERAGE => ($ENV{'LINTIAN_COVERAGE'}?1:0);
 
 use constant EMPTY => q{};
 use constant COLON => q{:};
+use constant NEWLINE => qq{\n};
 
 use Moo;
 use namespace::clean;
@@ -48,8 +53,7 @@ with 'Lintian::Check';
 
 has local_manpages => (is => 'rw', default => sub { {} });
 
-has running_man => (is => 'rw', default => sub { [] });
-has running_lexgrog => (is => 'rw', default => sub { [] });
+has running => (is => 'rw', default => sub { [] });
 
 sub spelling_tag_emitter {
     my ($self, @orig_args) = @_;
@@ -177,7 +181,7 @@ sub visit_installed_files {
             }
         }
     } else { # not a symlink
-        my $unpacked_path = $file->unpacked_path;
+
         my $fd;
         if ($file_info =~ m/gzip compressed/) {
             $fd = open_gz($file->unpacked_path);
@@ -220,6 +224,8 @@ sub visit_installed_files {
             }
         }
 
+        my $loop = IO::Async::Loop->new;
+
         # If it's not a .so link, use lexgrog to find out if the
         # man page parses correctly and make sure the short
         # description is reasonable.
@@ -231,19 +237,56 @@ sub visit_installed_files {
         # moment, leading to huge numbers of false negatives. When
         # man-db is fixed, this limitation should be removed.
         if ($path =~ m,/man/man\d/,) {
-            my $pid = open(my $lexgrog_fd, '-|');
-            if ($pid == 0) {
-                clean_env;
-                open(STDERR, '>&', \*STDOUT);
-                exec('lexgrog', $unpacked_path)
-                  or die "exec lexgrog failed: $!";
-            }
-            if (@{$self->running_lexgrog} > 2) {
-                $self->process_lexgrog_output($self->running_lexgrog);
-            }
-            # lexgrog can have a high start up time, so revisit
-            # this later.
-            push(@{$self->running_lexgrog}, [$file, $lexgrog_fd, $pid]);
+
+            my $future = $loop->new_future;
+            my $stdout;
+            my $stderr;
+
+            my $check = $self;
+
+            delete local $ENV{$_}
+              for grep { $_ ne 'PATH' && $_ ne 'TMPDIR' } keys %ENV;
+            local $ENV{LC_ALL} = 'C.UTF-8';
+
+            my @command = ('lexgrog', $file->unpacked_path);
+            my $process = IO::Async::Process->new(
+                command => [@command],
+                stdout => { into => \$stdout },
+                stderr => { into => \$stderr },
+                on_finish => sub {
+                    my ($self, $exitcode) = @_;
+                    my $status = ($exitcode >> 8);
+
+                    $check->tag('bad-whatis-entry', $file)
+                      if $status == 2;
+
+                    if ($status != 0 && $status != 2) {
+                        my $message = "Non-zero status $status from @command";
+                        $message .= COLON . NEWLINE . $stderr
+                          if length $stderr;
+                        $future->fail($message);
+                        return;
+                    }
+
+                    my $desc = $stdout;
+                    $desc =~ s/^[^:]+: \"(.*)\"$/$1/;
+
+                    if ($desc =~ /(\S+)\s+-\s+manual page for \1/i) {
+                        $check->tag('useless-whatis-entry', $file);
+
+                    } elsif ($desc =~ /\S+\s+-\s+programs? to do something/i) {
+                        $check->tag('manual-page-from-template', $file);
+                    }
+
+                    $future->done("Done with @command");
+                    return;
+                });
+
+            $loop->add($process);
+
+            $future->retain;
+
+            push(@{$self->running}, $future);
         }
 
         # If it's not a .so link, run it through 'man' to check for errors.
@@ -251,41 +294,110 @@ sub visit_installed_files {
         # parent directory before running man so that .so directives are
         # processed properly.  (Yes, there are man pages that include other
         # pages with .so but aren't simple links; rbash, for instance.)
-        my @cmd = qw(man --warnings -E UTF-8 -l -Tutf8 -Z);
-        my $dir;
-        if ($unpacked_path =~ m,^(.*)/(man\d/.*)$,) {
-            $dir = $1;
-            push @cmd, $2;
-        } else {
-            push(@cmd, $unpacked_path);
-        }
-        my ($read, $write);
-        pipe($read, $write);
-        my $pid = do_fork();
-        if ($pid == 0) {
-            clean_env;
-            close STDOUT;
-            close $read;
-            open(STDOUT, '>', '/dev/null');
-            open(STDERR, '>&', $write);
-            if ($dir) {
-                chdir($dir);
-            }
-            $ENV{MANROFFSEQ} = '';
-            $ENV{MANWIDTH} = 80;
-            exec { $cmd[0] } @cmd
-              or die "cannot run man -E UTF-8 -l: $!";
-        } else {
-            # parent - close write end
-            close $write;
-        }
+        {
+            my $future = $loop->new_future;
+            my $stdout;
+            my $stderr;
 
-        if (@{$self->running_man} > 3) {
-            $self->process_man_output($self->running_man);
+            my $check = $self;
+
+            delete local $ENV{$_}
+              for grep { $_ ne 'PATH' && $_ ne 'TMPDIR' } keys %ENV;
+            local $ENV{LC_ALL} = 'C.UTF-8';
+
+            local $ENV{MANROFFSEQ} = EMPTY;
+            local $ENV{MANWIDTH} = 80;
+
+            my @command = qw(man --warnings -E UTF-8 -l -Tutf8 -Z);
+
+            my $localdir = path($file->unpacked_path)->parent->stringify;
+            $localdir =~ s{^(.*)/man\d\b}{$1}s;
+
+            my $savedir = getcwd;
+            chdir($localdir);
+
+            push(@command, $file->unpacked_path);
+
+            my $process = IO::Async::Process->new(
+                command => [@command],
+                stdout => { into => \$stdout },
+                stderr => { into => \$stderr },
+                on_finish => sub {
+                    my ($self, $exitcode) = @_;
+                    my $status = ($exitcode >> 8);
+
+                    my @lines = split(/\n/, $stderr);
+
+                    for (@lines) {
+                        # Devel::Cover causes some annoying deep recursion
+                        # warnings and sometimes in our child process.
+                        # Filter them out, but only during coverage.
+                        next
+                          if LINTIAN_COVERAGE
+                          and m{
+                                                       \A Deep [ ] recursion [ ] on [ ] subroutine [ ]
+                                                       "[^"]+" [ ] at [ ] .*B/Deparse.pm [ ] line [ ]
+                                                       \d+}xsm;
+
+                        # ignore progress information from man
+                        next
+                          if /^Reformatting/;
+
+                        next
+                          if /^\s*$/;
+
+                  # ignore errors from gzip, will be dealt with at other places
+                        next
+                          if /^(?:man|gzip)/;
+
+                 # ignore wrapping failures for Asian man pages (groff problem)
+                        if ($language =~ /^(?:ja|ko|zh)/) {
+                            next
+                              if /warning \[.*\]: cannot adjust line/;
+                            next
+                              if /warning \[.*\]: can\'t break line/;
+                        }
+
+                     # ignore wrapping failures if they contain URLs (.UE is an
+                     # extension for marking the end of a URL).
+                        next
+                          if
+/:(\d+): warning \[.*\]: (?:can\'t break|cannot adjust) line/
+                          and( $manfile[$1 - 1] =~ m,(?:https?|ftp|file)://.+,i
+                            or $manfile[$1 - 1] =~ m,^\s*\.\s*UE\b,);
+
+                     # ignore common undefined macros from pod2man << Perl 5.10
+                        next
+                          if /warning: (?:macro )?\'(?:Tr|IX)\' not defined/;
+
+                        chomp;
+                        s/^[^:]+: //;
+                        s/^<standard input>://;
+
+                        $check->tag('groff-message', $file, $_);
+
+                        last;
+                    }
+
+                    if ($status) {
+                        my $message = "Non-zero status $status from @command";
+                        $message .= COLON . NEWLINE . $stderr
+                          if length $stderr;
+                        $future->fail($message);
+                        return;
+                    }
+
+                    $future->done("Done with @command");
+                    return;
+                });
+
+            $loop->add($process);
+            $future->retain;
+
+            chdir($savedir);
+
+            push(@{$self->running}, $future);
         }
-        # man can have a high start up time, so revisit this
-        # later.
-        push(@{$self->running_man},[$file, $read, $pid, $language, \@manfile]);
 
         # Now we search through the whole man page for some common errors
         my $lc = 0;
@@ -488,80 +600,15 @@ sub breakdown_installed_files {
           if $section == 1 || $section == 8;
     }
 
-    # If we have any running sub processes, wait for them here.
-    $self->process_lexgrog_output($self->running_lexgrog)
-      if @{$self->running_lexgrog};
-    $self->process_man_output($self->running_man) if @{$self->running_man};
+    Future->wait_all(@{$self->running})->get;
 
-    return;
-}
-
-sub process_lexgrog_output {
-    my ($self, $running) = @_;
-    for my $lex_proc (@{$running}) {
-        my ($file, $lexgrog_fd, undef) = @{$lex_proc};
-        my $desc = <$lexgrog_fd>;
-        $desc =~ s/^[^:]+: \"(.*)\"$/$1/;
-        if ($desc =~ /(\S+)\s+-\s+manual page for \1/i) {
-            $self->tag('useless-whatis-entry', $file);
-        } elsif ($desc =~ /\S+\s+-\s+programs? to do something/i) {
-            $self->tag('manual-page-from-template', $file);
-        }
-        drain_pipe($lexgrog_fd);
-        eval {close($lexgrog_fd);};
-        if (my $err = $@) {
-            # Problem closing the pipe?
-            die "close pipe: $err: $!"
-              if $err->errno;
-            # No, then lexgrog returned with a non-zero exit code.
-            $self->tag('bad-whatis-entry', $file);
-        }
+    for my $future (@{$self->running}) {
+        warn $future->failure
+          if $future->is_failed;
     }
-    @{$running} = ();
-    return;
-}
 
-sub process_man_output {
-    my ($self, $running) = @_;
-    for my $man_proc (@{$running}) {
-        my ($file, $read, $pid, $language, $contents) = @{$man_proc};
-        while (<$read>) {
-            # Devel::Cover causes some annoying deep recursion
-            # warnings and sometimes in our child process.
-            # Filter them out, but only during coverage.
-            next if LINTIAN_COVERAGE and m{
-                    \A Deep [ ] recursion [ ] on [ ] subroutine [ ]
-                    "[^"]+" [ ] at [ ] .*B/Deparse.pm [ ] line [ ]
-                   \d+}xsm;
-            # ignore progress information from man
-            next if /^Reformatting/;
-            next if /^\s*$/;
-            # ignore errors from gzip, will be dealt with at other places
-            next if /^(?:man|gzip)/;
-            # ignore wrapping failures for Asian man pages (groff problem)
-            if ($language =~ /^(?:ja|ko|zh)/) {
-                next if /warning \[.*\]: cannot adjust line/;
-                next if /warning \[.*\]: can\'t break line/;
-            }
-            # ignore wrapping failures if they contain URLs (.UE is an
-            # extension for marking the end of a URL).
-            next
-              if/:(\d+): warning \[.*\]: (?:can\'t break|cannot adjust) line/
-              and ($contents->[$1 - 1] =~ m,(?:https?|ftp|file)://.+,i
-                or $contents->[$1 - 1] =~ m,^\s*\.\s*UE\b,);
-            # ignore common undefined macros from pod2man << Perl 5.10
-            next if /warning: (?:macro )?\'(?:Tr|IX)\' not defined/;
-            chomp;
-            s/^[^:]+: //;
-            s/^<standard input>://;
-            $self->tag('groff-message', $file, $_);
-            last;
-        }
-        close($read);
-        # reap man process
-        waitpid($pid, 0);
-    }
-    @{$running} = ();
+    @{$self->running} = ();
+
     return;
 }
 
