@@ -75,7 +75,7 @@ sub build_path {
     return EMPTY
       unless $buildinfo;
 
-    return $buildinfo->fields->value('Build-Path') // EMPTY;
+    return $buildinfo->fields->value('Build-Path');
 }
 
 sub check_item {
@@ -91,8 +91,7 @@ sub check_item {
           if $item->mentions_in_operation($SENSIBLE_REGEX);
     }
 
-    unless (($self->processable->fields->value('Section') // EMPTY) eq
-        'debian-installer'
+    unless ($self->processable->fields->value('Section') eq 'debian-installer'
         || any { $_ eq $self->processable->source } qw(base-files dpkg)) {
 
         $self->tag('uses-dpkg-database-directly', $item->name)
