@@ -62,7 +62,7 @@ use Path::Tiny;
 use POSIX qw(locale_h strftime);
 
 use Lintian::Data;
-use Lintian::Deb822Parser qw(read_dpkg_control);
+use Lintian::Deb822::Parser qw(read_dpkg_control_lc);
 use Lintian::Profile;
 
 =head1 FUNCTIONS
@@ -141,7 +141,7 @@ sub get_required_debhelper_version {
     die 'Cannot get latest version of debhelper from debian/control'
       unless -f $controlfile;
 
-    my @paragraphs = read_dpkg_control($controlfile);
+    my @paragraphs = read_dpkg_control_lc($controlfile);
     die "$controlfile does not have even one paragraph"
       if (scalar(@paragraphs) < 1);
 
